/**
 * (C) Copyright IBM Corp. 2007, 2009
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE ECLIPSE PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Eclipse Public License from
 * http://www.opensource.org/licenses/eclipse-1.0.php
 *
 * @author : Endre Bak, IBM, ebak@de.ibm.com
 * 
 * Change History
 * Flag       Date        Prog         Description
 *------------------------------------------------------------------------------- 
 * 1804402    2007-09-28  ebak         IPv6 ready SLP
 * 2003590    2008-06-30  blaschke-oss Change licensing from CPL to EPL
 * 2204488 	  2008-10-28  raman_arora  Fix code to remove compiler warnings
 * 2524131    2009-01-21  raman_arora  Upgrade client to JDK 1.5 (Phase 1)
 * 2531371    2009-02-10  raman_arora  Upgrade client to JDK 1.5 (Phase 2)
 * 2763216    2009-04-14  blaschke-oss Code cleanup: visible spelling/grammar errors
 * 2823494    2009-08-03  rgummada     Change Boolean constructor to static
 */

package org.sblim.cimclient.unittest.slp.internal;

import java.util.Vector;

import org.sblim.cimclient.unittest.GenericUTestExts;
import org.sblim.cimclient.unittest.TestCase;
import org.sblim.slp.ServiceLocationAttribute;
import org.sblim.slp.internal.AttributeHandler;
import org.sblim.slp.internal.Convert;

/**
 * ConvertTest
 * 
 */
public class ConvertTest extends TestCase {

	private static final String[] RAW_STRINGS = { "(hi,joe)", "hello~(hi,joe!) world" };

	private static final String[] ESCAPED_STRINGS = { "\\28hi\\2cjoe\\29",
			"hello\\7e\\28hi\\2cjoe\\21\\29 world" };

	/**
	 * testEscaping
	 * 
	 * @throws Exception
	 */
	public void testEscaping() throws Exception {
		for (int i = 0; i < RAW_STRINGS.length; i++) {
			String rawStr = RAW_STRINGS[i];
			String escapedStr = Convert.escape(rawStr);
			String refEscapedStr = ESCAPED_STRINGS[i];
			verify("Escaping failure!\n\"" + rawStr + "\" -> \"" + escapedStr + "\" != \""
					+ refEscapedStr + " !", refEscapedStr.equals(escapedStr));
			String unescapedStr = Convert.unescape(escapedStr);
			verify("Unescaping failure!\n\"" + escapedStr + "\" -> \"" + unescapedStr + "\" != \""
					+ rawStr + " !", rawStr.equals(unescapedStr));
		}
	}

	private static final Object[][] ATTR_VALS_ARRAY = {
			null,
			new Object[] { "String (o) Value", Boolean.TRUE, Boolean.FALSE,
					new byte[] { 0x20, 0x15, 0x33, 0x5a }, new Integer(42) },
			new Object[] { new byte[] { (byte) 0xff, (byte) 0xef, 0x5a }, "Hello*!" },
			new Object[] { "Just a single string.(,)" } };

	private static Vector<Object> getAttrValues(int pIdx) {
		return GenericUTestExts.mkVec(ATTR_VALS_ARRAY[pIdx]);
	}

	private static ServiceLocationAttribute getAttrib(int pIdx) {
		return new ServiceLocationAttribute("(A)ttrib" + pIdx, getAttrValues(pIdx));
	}

	/**
	 * testAttribStringHandling
	 * 
	 * @throws Exception
	 */
	public void testAttribStringHandling() throws Exception {
		for (int i = 0; i < ATTR_VALS_ARRAY.length; i++) {
			ServiceLocationAttribute refAttrib = getAttrib(i);
			String refAttrStr = AttributeHandler.buildString(refAttrib);
			debug(refAttrStr);
			ServiceLocationAttribute attrib = new ServiceLocationAttribute(refAttrStr);
			String attrStr = AttributeHandler.buildString(attrib);
			debug(attrStr);
			verify("Rebuilt attribute doesn't equal the reference one!", refAttrib.equals(attrib));
			verify("Rebuilt attribute string doesn't equal the original one!", refAttrStr
					.equals(attrStr));
		}
	}

	/**
	 * @param pMsg
	 */
	private static void debug(String pMsg) {
	// System.out.println(pMsg);
	}

}
